/*
 * Copyright (c) 2004 by Tsuyoshi Iguchi (zinnia@risky-safety.org)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: lua_context.c,v 1.9 2004/04/30 04:01:39 zinnia Exp $
 */

#include "sps.h"

#define LUA_FUNCTION_NAME(name) luafunction_##name
#define LUA_FUNCTION(name) static int LUA_FUNCTION_NAME(name) (lua_State* L)

#define REGISTER_LUA_FUNCTION(lua,name) lua_register(lua, "sps_" #name, LUA_FUNCTION_NAME(name))

// ----------
#define LUA_FUNCTION_GETCONTEXT(c) SPS_Render_Context* c = SPS_Render_Context_Lookup(L)

LUA_FUNCTION(pause)
{
	int timer = -1;
	LUA_FUNCTION_GETCONTEXT(context);

	if(lua_gettop(L) >= 1) {
		timer = lua_tonumber(L, 1);
	}

	SPS_Pause(context, timer);
	return 0;
}

LUA_FUNCTION(movecursor)
{
	LUA_FUNCTION_GETCONTEXT(context);
	if(lua_gettop(L) >= 2) {
		SPS_MoveCursor(context, lua_tonumber(L, 1), lua_tonumber(L, 2));
	}
	return 0;
}

LUA_FUNCTION(puttext)
{
	LUA_FUNCTION_GETCONTEXT(context);
	if(lua_gettop(L) >= 1){
		const char* fontindex = "0";
		SPS_VERTICAL_ALIGN align = SPS_VERTICAL_ALIGN_BOTTOM;

		if(lua_gettop(L) >= 2){
			fontindex = lua_tostring(L, 2);
		}
		if(lua_gettop(L) >= 3){
			align = SPS_Vertical_Align_Parse(lua_tostring(L, 3));
		}
		SPS_PutText(context, lua_tostring(L, 1), fontindex, align);
	}
	return 0;
}

LUA_FUNCTION(bg)
{
	LUA_FUNCTION_GETCONTEXT(context);
	if(lua_gettop(L) >= 1){
		int alpha = 100;
		SPS_BGSTYLE style = SPS_BGSTYLE_TILE;
		if(lua_gettop(L) >= 2){
			style = SPS_BgStyle_Parse(lua_tostring(L, 2));
			if(lua_gettop(L) >= 3){
				alpha = lua_tonumber(L, 3);
			}
		}
		SPS_Bg(context, lua_tostring(L, 1), style, alpha);
	}
	return 0;
}

LUA_FUNCTION(deffont)
{
	LUA_FUNCTION_GETCONTEXT(context);
	if(lua_gettop(L) >= 2){
		const char* path;
		const char* name;

		name = lua_tostring(L, 1);
		path = lua_tostring(L, 2);

		SPS_DefFont(context, name, path);
	}
	return 0;
}

LUA_FUNCTION(deffontindex)
{
	LUA_FUNCTION_GETCONTEXT(context);
	if(lua_gettop(L) >= 3){
//		char* oldkey; void* oldvalue;
		const char* name;
		const char* id;
		int point;
		int aa = 0;

		id = lua_tostring(L, 1);
		name = lua_tostring(L, 2);
		point = RatioToPixel(context, lua_tonumber(L, 3));
		if(lua_gettop(L) >= 4){
			aa = lua_tonumber(L, 4);
		}
		SPS_DefFontIndex(context, id, name, point, aa);
	}
	return 0;
}

LUA_FUNCTION(forecolor)
{
	LUA_FUNCTION_GETCONTEXT(context);
	if(lua_gettop(L) >= 1){
		SPS_ForeColor(context, lua_tostring(L, 1));
	}
	return 0;
}
LUA_FUNCTION(backcolor)
{
	LUA_FUNCTION_GETCONTEXT(context);
	if(lua_gettop(L) >= 1){
		SPS_BackColor(context, lua_tostring(L, 1));
	}
	return 0;
}

LUA_FUNCTION(puthbar)
{
	LUA_FUNCTION_GETCONTEXT(context);
	if(lua_gettop(L) >= 1)
	{
		SPS_PutHBar(context, lua_tonumber(L, 1));
	}
	return 0;
}

LUA_FUNCTION(moveto)
{
	LUA_FUNCTION_GETCONTEXT(context);
	if(lua_gettop(L) >= 2)
	{
		SPS_MoveTo(context, lua_tonumber(L, 1), lua_tonumber(L, 2));
	}
	return 0;

}
LUA_FUNCTION(beginline)
{
	LUA_FUNCTION_GETCONTEXT(context);
	const char* halign = "";
	if(lua_gettop(L) >= 1){
		halign = lua_tostring(L, 1);
	}
	SPS_BeginLine(context, halign);
	return 0;
}
LUA_FUNCTION(endline)
{
	LUA_FUNCTION_GETCONTEXT(context);
	SPS_EndLine(context);
	return 0;
}

LUA_FUNCTION(fontstyle)
{
	LUA_FUNCTION_GETCONTEXT(context);
	if(lua_gettop(L) >= 2){
		SPS_FontStyle(context, lua_tostring(L, 1), lua_tostring(L, 2));
	}
	return 0;
}

LUA_FUNCTION(hmargin)
{
	LUA_FUNCTION_GETCONTEXT(context);
	if(lua_gettop(L) >= 1){
		SPS_HMargin(context, lua_tonumber(L, 1));
	}
	return 0;
}
LUA_FUNCTION(deficon)
{
	LUA_FUNCTION_GETCONTEXT(context);
	if(lua_gettop(L) >= 2){
		SPS_DefIcon(context, lua_tostring(L, 1), lua_tostring(L, 2));
	}
	return 0;
}

LUA_FUNCTION(puticon)
{
	LUA_FUNCTION_GETCONTEXT(context);
	if(lua_gettop(L) >= 1){
		SPS_VERTICAL_ALIGN align = SPS_VERTICAL_ALIGN_BOTTOM;
		int width, height;
		width = height = -1;

		if(lua_gettop(L) >= 2){
			align = SPS_Vertical_Align_Parse(lua_tostring(L, 2));
			if(lua_gettop(L) >= 3){
				width = RatioToPixel(context, lua_tonumber(L, 3));
				if(lua_gettop(L) >= 4){
					height = RatioToPixel(context, lua_tonumber(L, 4));
				}
			}
		}
		SPS_PutIcon(context, lua_tostring(L, 1), align, width, height);
	}
	return 0;
}

LUA_FUNCTION(putimage)
{
	LUA_FUNCTION_GETCONTEXT(context);
	if(lua_gettop(L) >= 1){
		SPS_VERTICAL_ALIGN align = SPS_VERTICAL_ALIGN_BOTTOM;
		int width, height;
		width = height = -1;
		if(lua_gettop(L) >= 2){
			align = SPS_Vertical_Align_Parse(lua_tostring(L, 2));
			if(lua_gettop(L) >= 3){
				width = RatioToPixel(context, lua_tonumber(L, 3));
				if(lua_gettop(L) >= 4){
					height = RatioToPixel(context, lua_tonumber(L, 4));
				}
			}
		}
		SPS_PutImage(context, lua_tostring(L, 1), align, width, height);
	}
	return 0;
}

LUA_FUNCTION(beginpage){ 
	LUA_FUNCTION_GETCONTEXT(context);
	const char* title = "";

	if(lua_gettop(L) >= 1){
		title = lua_tostring(L, 1);
	}
	SPS_BeginPage(context, title);

	return 0;
}

LUA_FUNCTION(endpage){
	LUA_FUNCTION_GETCONTEXT(context);
	SPS_EndPage(context);
	return 0;
}

LUA_FUNCTION(flip_function){
	LUA_FUNCTION_GETCONTEXT(context);
	if(lua_gettop(L) >= 1){
		SPS_Flip_Function(context, lua_tostring(L, 1));
	}
	return 0;
}
LUA_FUNCTION(title)
{
	LUA_FUNCTION_GETCONTEXT(context);
	if(lua_gettop(L) >= 1){
		SPS_SetTitle(context, lua_tostring(L, 1));
	}
	return 0;
}

void lua_context_prepare(lua_State* L)
{
	REGISTER_LUA_FUNCTION(L, title);
	REGISTER_LUA_FUNCTION(L, pause);
	REGISTER_LUA_FUNCTION(L, movecursor);
	REGISTER_LUA_FUNCTION(L, puttext);

	REGISTER_LUA_FUNCTION(L, deffont);
	REGISTER_LUA_FUNCTION(L, deffontindex);

	REGISTER_LUA_FUNCTION(L, forecolor);
	REGISTER_LUA_FUNCTION(L, backcolor);

	REGISTER_LUA_FUNCTION(L, puthbar);
	REGISTER_LUA_FUNCTION(L, moveto);
	REGISTER_LUA_FUNCTION(L, beginline);
	REGISTER_LUA_FUNCTION(L, endline);
	REGISTER_LUA_FUNCTION(L, fontstyle);
	
	REGISTER_LUA_FUNCTION(L, hmargin);
	REGISTER_LUA_FUNCTION(L, deficon);
	REGISTER_LUA_FUNCTION(L, puticon);

	REGISTER_LUA_FUNCTION(L, putimage);
	REGISTER_LUA_FUNCTION(L, bg);

	REGISTER_LUA_FUNCTION(L, beginpage);
	REGISTER_LUA_FUNCTION(L, endpage);
	REGISTER_LUA_FUNCTION(L, flip_function);
}
