/*
 * Copyright (c) 2004 by Tsuyoshi Iguchi (zinnia@risky-safety.org)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	arch_tag:
 */

#include "sps.h"
#include "sps-displaybase.h"
#include "sps-displaypresentation.h"
#include "sps-displayloading.h"

//#undef PARSE_THREAD

SPS_Display* display;

SPS_Flip_Parameter* SPS_Flip_Parameter_new(
	SPS_Presentation* presentation,
	SPS_Scene* scene_before,
	SPS_Scene* scene_after)
{
	SPS_Flip_Parameter* result = g_malloc0(sizeof(SPS_Flip_Parameter));
	if(result != NULL){
		result->tick_begin = SDL_GetTicks();
		result->tick_current = result->tick_begin;
		result->userdata = NULL;
		result->scene_before = scene_before;
		result->scene_after = scene_after;
	}
	return result;
}

void SPS_FPS_Reset()
{
	display->tick_begin = SDL_GetTicks();
	display->frames = 0;
	display->fps = 0;
}

void SPS_TextureCount_Reset()
{
	display->textures = 0;
}


// XXX
SPS_Display* SPS_Display_new()
{
	return g_malloc0(sizeof(SPS_Display));
}

/* XXX not called */
static void destroy()
{
	/* XXX release contents */
	
#if 0
	g_hash_table_destroy(context->icons);
	g_hash_table_destroy(context->fontpath);
	g_hash_table_destroy(context->font);

	SDL_FreeSurface(context->surface);
#endif

}

static void parse_geometry(const char* expression, int* w, int* h)
{
	sscanf(expression, "%dx%d", w, h);
	return;
}

//#define MAXFPS (60)
//#define DISPLAY_DELAY (1000 / (MAXFPS))

// $B%U%!%$%k%Q!<%9MQ%9%l%C%I(B
static int parsing_thread(void* data)
{
	SDL_Event e;
	SPS_Render_Context* context = data;
	SPS_Page_Parse_All(context);
//	while(!g_queue_is_empty(display->queue)){
//		SDL_Delay(100);
//	}
	e.type = SDL_USEREVENT;
	e.user.code = SPS_USEREVENT_PARSE_DONE;

	// $BJL%9%l%C%I$+$i%$%Y%s%H$r(BPush$B$9$k$N$O$^$:$$$s$@$m$&$J$"(B
	SDL_PushEvent(&e);
	return 0;
}

int main(int argc, char* argv[])
{
	int i;

	char* path = NULL;
	SPS_Render_Context* context;
	display = SPS_Display_new();
	
	display->fullscreen = 0;
	display->w = 800;
	display->h = 600;

	display->sprite_shade = TRUE;
	display->showfps = 1;
#ifdef USE_OPENGL
	display->use_opengl = TRUE;
#endif

	for(i = 1; i < argc; i++){
		switch(*argv[i]){
		case '-':
			switch(*(argv[i] + 1)){
			case 'h':
			case '?':
				usage();
				break;

			case 'g':
				i++;
				parse_geometry(argv[i],
					       &(display->w),
					       &(display->h));
				break;
			case 'f':
				display->fullscreen = 1 - display->fullscreen;
				break;
			case 'F':
				display->showfps = 0;
				break;
#ifdef USE_OPENGL
			case 'G':
				display->use_opengl = FALSE;
				break;
#endif
			}
			break;
		default:
			path = argv[i];
			break;
		}
	}
	if(path == NULL) usage();

	g_type_init();

	if(SDL_Init(SDL_INIT_VIDEO|SDL_INIT_NOPARACHUTE) < 0){
		g_error("SDL_Init: %s", SDL_GetError());
		return 0;
	}
	if(TTF_Init() < 0){
		g_error("TTF_Init: %s", SDL_GetError());
		return 0;
	}

#ifdef USE_OPENGL
    SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 5 );
    SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 5 );
    SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 5 );
    SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16 );
    SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
#endif
	atexit(SDL_Quit);


#ifdef USE_OPENGL
#define ADDITIONAL_FLAGS SDL_OPENGL
#else
#define ADDITIONAL_FLAGS SDL_SWSURFACE
#endif

	display->display = 
		SDL_SetVideoMode(display->w, 
				 display->h, 
				 32,
				 ADDITIONAL_FLAGS | 
				 (display->fullscreen ? SDL_FULLSCREEN : 0));

	if(display->display == NULL){
		g_error("SDL_SetVideoMode: %s", SDL_GetError());
		return 0;
	}


	SPS_FPS_Reset();

	display->format = display->display->format; // XXX
	display->pageno = 1;

#ifdef USE_OPENGL
	setup_opengl();
#endif

	context = SPS_Open_File(path, FALSE);
	g_debug("%d pages.", context->presentation->pageno_max);

	// Loading System Font
	SPS_DefFont(context, "SYSTEM", SYSTEM_FONT);
	SPS_DefFontIndex(context, "SYSTEM8", "SYSTEM",
			 RatioToPixel(context, 8), 0);
	SPS_DefFontIndex(context, "SYSTEM6", "SYSTEM",
			 RatioToPixel(context, 6), 0);
	SPS_DefFontIndex(context, "SYSTEM4", "SYSTEM",
			 RatioToPixel(context, 4), 0);
	

	if(context->presentation->pageno_max <= 0){
		fprintf(stderr, "%s: No pages to show.\n", path);
		return 1;
	}

	display->queue = g_queue_new();
	display->sprite_count = 0;

	display->tick_begin = SDL_GetTicks();
	display->frames = 0;

#ifdef PARSE_THREAD
	context->thread = SDL_CreateThread(parsing_thread, (void*)context);
#else
	parsing_thread((void*)context);
#endif

	SPS_DisplayManager_Init(context);

#ifdef PARSE_THREAD
	SpsDisplayLoading* l = g_object_new(SPS_TYPE_DISPLAYLOADING, NULL);
	sps_displayloading_set_render_context(l, context);

	SPS_DisplayManager_AppendWindow(NULL, SPS_DISPLAYBASE(l));
	sps_displaybase_show(SPS_DISPLAYBASE(l));
#else
	display->scene_last.page = NULL;
	display->scene_current.page = SPS_Page_Get(context->presentation, display->pageno);
	display->scene_current.pauselevel = 0;

	display->flip_driver = display->scene_current.page->setting.flip_driver;
	display->flip_parameter = SPS_Flip_Parameter_new(
		context->presentation,
		&display->scene_last, &display->scene_current);

	SpsDisplayPresentation* dp = g_object_new(SPS_TYPE_DISPLAYPRESENTATION, NULL);
	SPS_DisplayManager_AppendWindow(NULL, SPS_DISPLAYBASE(dp));
	sps_displaybase_show(SPS_DISPLAYBASE(dp));
	sps_displaypresentation_set_presentation(dp, context->presentation);
#endif

	SPS_DisplayManager_MainLoop();
	return 0;
}
