/*
 * Copyright (c) 2004 by Tsuyoshi Iguchi (zinnia@risky-safety.org)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: flip_typoh.c,v 1.6 2004/05/18 03:49:36 zinnia Exp $
 */
#include "sps.h"
static const int ms = 1000;

static void doprocess(SPS_Flip_Parameter* param, SPS_Scene* scene, Uint32 tick_diff, gboolean before)
{
	int ofs, lastline;
	GList* p;
	float ratio =  (float)tick_diff / (float)ms;

	float alpha = before ? (1.0 - ratio) : ratio;
	int offset = before ? tick_diff : (tick_diff - ms);
//	glColor4f(1.0, 1.0, 1.0, alpha);

	SPS_Page_Draw_Bg(scene->page, alpha);

	lastline = -1;
	G_LIST_FOREACH(scene->page->drawunits, p){
		SPS_DrawUnit* u = (SPS_DrawUnit*)p->data;
		if(u == NULL) continue;
		if(lastline != u->lineno){
			ofs = 0;
			lastline = u->lineno;
		}
		if(u->pauselevel <= scene->pauselevel){
			SPS_Draw_Parameter dparam;
			dparam.alpha = alpha;
			dparam.shade = TRUE;
			dparam.x = u->sprite.x + offset  * (u->lineno % 2 ? -1 : +1) - display->w / 2;
			dparam.y =  u->sprite.y - display->h / 2;
			SPS_Sprite_Bind(&u->sprite);
			SPS_Sprite_Draw_Generic(&u->sprite,
						     SPS_DRAW_PARAMETER_ENUM_POSITION |
						     SPS_DRAW_PARAMETER_ENUM_ALPHA,
						     &dparam						     
						    );

		}
		ofs++;
	}
}

static gboolean Process(SPS_Flip_Parameter* param)
{
	Uint32 tick_diff = param->tick_current - param->tick_begin;
	if(tick_diff > ms) tick_diff = ms;

	if(param->scene_before->page != NULL){
		doprocess(param, param->scene_before, tick_diff, TRUE);
	}
	if(param->scene_after->page != NULL){
		doprocess(param, param->scene_after, tick_diff, FALSE);
	}
	if(tick_diff == ms) return FALSE;
	return TRUE; 
}

SPS_Flip_Vector typoh_vector = {
	FALSE, // require_opengl
	"TypoH", // name
	"Typo (Horizontal)", // description
	TRUE, // disappear
	TRUE, // appear
	Process,
};
