/*
 * Copyright (c) 2004 by Tsuyoshi Iguchi (zinnia@risky-safety.org)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: sps.h,v 1.19 2004/05/18 03:49:36 zinnia Exp $
 */

#ifndef _SPS_H
#define _SPS_H

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "SDL.h"
#include "SDL_thread.h"

#include "SDL_image.h"

#include "SDL_ttf.h"

#define USE_OPENGL
#define PARSE_THREAD
//#undef USE_OPENGL
#ifdef USE_OPENGL
#include <SDL_opengl.h>
#endif

#include <lua.h>
#include <lualib.h>
#include <lauxlib.h>
//#include <iconv.h>

#include <ctype.h>
#include <string.h>

#include <glib.h>

#define G_LOG_DOMAIN "Sps"   // XXX ????
#ifndef g_debug
#define g_debug(...) g_log(G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, __VA_ARGS__)
#endif

//#include "SDL_kanji.h"

#ifndef G_LIST_FOREACH
#define G_LIST_FOREACH(list,p) for(p = g_list_first(list); p != NULL; p = g_list_next(p))
#endif


#define DEFAULT_ENCODING "EUC-JP"
#define SPS_SYSTEM_CONFIG_PATH "system.lua"

// --------------------

// $B%l%s%@%j%s%00LCV9g$o$;4X78(B
typedef enum {
	SPS_VERTICAL_ALIGN_TOP,
	SPS_VERTICAL_ALIGN_CENTER,
	SPS_VERTICAL_ALIGN_BOTTOM,
} SPS_VERTICAL_ALIGN;

typedef enum {
	SPS_HORIZONTAL_ALIGN_LEFT,
	SPS_HORIZONTAL_ALIGN_CENTER,
	SPS_HORIZONTAL_ALIGN_RIGHT,
} SPS_HORIZONTAL_ALIGN;

// $BGX7J%9%?%$%k(B
typedef enum {
	SPS_BGSTYLE_TILE,
	SPS_BGSTYLE_ZOOM,
	SPS_BGSTYLE_CENTER,
} SPS_BGSTYLE;

// $BJ8;z(B(LETTER)/$B%"%$%3%s(B(PRIMITIVE)$B$N<oJL(B
typedef enum {
	SPS_DRAWUNIT_TYPE_PRIMITIVE,
	SPS_DRAWUNIT_TYPE_LETTER,
} SPS_DRAWUNIT_TYPE;

// $B%9%W%i%$%H(B($BJ8;z!"%"%$%3%s$J$I$N85$K$J$k$b$N(B)
typedef struct {
	/* texture / surface */
	SDL_Surface* surface;
	gboolean surface_dirty;

#ifdef USE_OPENGL
	GLuint texture;
	GLfloat texcoord_minx, texcoord_maxx, texcoord_miny, texcoord_maxy;
	gboolean texture_autofree;
//	int texw, texh;
#endif
	gboolean surface_autofree;

	/* draw setting */
	int x, y; /* in pixel */
	int w, h;

	/* user data */
	gpointer* userdata;
	gboolean userdata_autofree;
} SPS_Sprite;

// $B%9%W%i%$%HJ];}MQ%G!<%?(B
typedef struct {
	SPS_Sprite sprite;
	int pauselevel; // $BI=<($r3+;O$9$k%]!<%:%l%Y%k(B
	int lineno;
	SPS_DRAWUNIT_TYPE type;

	SPS_VERTICAL_ALIGN valign; // XXX messy
} SPS_DrawUnit;

struct _SPS_Flip_Vector;
typedef struct _SPS_Flip_Vector SPS_Flip_Vector;


// $B%Z!<%8@_Dj(B
typedef struct {
	SPS_BGSTYLE bg_style;
	SPS_Sprite* bg_sprite;
	double bg_alpha;

	SPS_Flip_Vector* flip_driver;

	Uint32 backcolor;
	Uint32 forecolor;
	SPS_HORIZONTAL_ALIGN halign;
	int hmargin;

} SPS_PageSetting;


	
// $B%Z!<%8J];}MQ%G!<%?(B
typedef struct {
	GList* drawunits; // array of SPS_DrawUnit
	int pauselevel_max; // $B:GBg%]!<%:%l%Y%k(B

//	char* source; // $B%=!<%9%U%!%$%k(B
	GString* source;

	gchar* title;

	SPS_PageSetting setting;

#ifdef USE_OPENGL
	GLuint displaylist; // $B%G%#%9%W%l%$%j%9%H(B
#endif
} SPS_Page;

typedef struct {
	SPS_Page* page;
	int pauselevel;
} SPS_Scene;

// $B%W%l%<%sA4BN(B
typedef struct {
	int pageno_max;
	GList* pages;
	gchar* title;
} SPS_Presentation;


typedef struct {
	SPS_Presentation* presentation;
	SPS_Scene* scene_before;
	SPS_Scene* scene_after;

	Uint32 tick_begin;
	Uint32 tick_current;

	gpointer* userdata;
} SPS_Flip_Parameter;



struct _SPS_Flip_Vector {
	gboolean require_opengl;
	char* name;
	char* description;
	gboolean disapper;
	gboolean appear;

	gboolean (*Process)(SPS_Flip_Parameter* p);
} ;

// $B2hLLB&(B
typedef struct {
	SPS_Presentation* presentation; // XXX GList*

	// FPS$B4IM}(B
	Uint32 tick_begin;
	Uint32 frames;
	Uint32 textures;
	int fps;

	gboolean showfps;
	gboolean fullscreen;
	gboolean use_opengl;

	int w, h;


	SDL_PixelFormat* format;

	SDL_Surface* display;

//	int pauselevel;
	int pageno;

	SPS_Scene scene_current;
	SPS_Scene scene_last;

	SPS_Flip_Vector* flip_driver;
	SPS_Flip_Parameter* flip_parameter;

	gboolean sprite_shade;

//	Kanji_Font* kanji_font;

	GQueue* queue;
	int sprite_count;

} SPS_Display;
//#define KANJI_FONT_FILENAME "10x20-ISO8859-1.bdf"
//#define KANJI_FONT_SIZE 20
//#define KANJI_FONT_FILENAME "6x12.bdf"
//#define KANJI_FONT_SIZE 12

#define SYSTEM_FONT "kochi.ttf"


// $B%l%s%@%j%s%0;~$K;HMQ$9$k%G!<%?(B
typedef struct {
	SPS_Presentation* presentation;

	const char* path;

	lua_State* lua;
	int pauselevel;
	int lineno;

	int pageno;

	SPS_Page* page_current;

	GList* line;

	int in_linerender;
	int in_pagerender;

	SDL_Rect cursor;

	SPS_PageSetting setting;

	GHashTable* icons; // icon table: name -> surface */
	GHashTable* fontpath; // name -> font path */
	GHashTable* fonts; // number -> TTF_Font* */

	int sprite_count_max;

	SDL_Thread* thread; // reading thread
} SPS_Render_Context;

typedef enum
{
	SPS_DRAW_PARAMETER_ENUM_NONE = 0,
	SPS_DRAW_PARAMETER_ENUM_POSITION = 1,
	SPS_DRAW_PARAMETER_ENUM_ALPHA = 2,
} SPS_DRAW_PARAMETER_ENUM;

typedef struct {
	int x;
	int y;
	double alpha;
	gboolean shade;
} SPS_Draw_Parameter ;

extern void lua_context_prepare(lua_State* L);

extern SPS_VERTICAL_ALIGN SPS_Vertical_Align_Parse(const char* expression);
extern SPS_HORIZONTAL_ALIGN SPS_Horizontal_Align_Parse(const char* expression);
extern SPS_BGSTYLE SPS_BgStyle_Parse(const char* expression);

extern void SPS_Fill(SPS_Render_Context* context);
extern void SPS_Pause(SPS_Render_Context* context, int timer);
extern void SPS_MoveCursor(SPS_Render_Context* context, int x, int y);
extern void SPS_PutText(SPS_Render_Context* context, const char* text, const char* fontindex, SPS_VERTICAL_ALIGN align);

extern void SPS_ForeColor(SPS_Render_Context* context, const char* color);
extern void SPS_BackColor(SPS_Render_Context* context, const char* color);
extern void SPS_PutHBar(SPS_Render_Context* context, double thick);

extern void SPS_MoveTo(SPS_Render_Context* context, int x, int y);
extern void SPS_EndLine(SPS_Render_Context* context);
extern void SPS_BeginLine(SPS_Render_Context* context, const char* halign);
extern void SPS_FontStyle(SPS_Render_Context* context, const char* fontindex, const char* expression);
extern void SPS_DefFont(SPS_Render_Context* context, const char* name, const char* path);
extern void SPS_DefFontIndex(SPS_Render_Context* context, const char* id, const char* name, int point, int aa);

extern void SPS_HMargin(SPS_Render_Context* context, int m);

extern void SPS_DefIcon(SPS_Render_Context* context, const char* no, const char* filename);
extern void SPS_PutIcon(SPS_Render_Context* context, const char* iconindex, SPS_VERTICAL_ALIGN align, int width, int height);
extern void SPS_PutImage(SPS_Render_Context* context, const char* path, SPS_VERTICAL_ALIGN align, int width, int height);
extern void SPS_Bg(SPS_Render_Context* context, const char* path, SPS_BGSTYLE style, int alpha);

extern void SPS_BeginPage(SPS_Render_Context* context, char* title);
extern void SPS_EndPage(SPS_Render_Context* context);
extern void SPS_Include(SPS_Render_Context* context, const char* filename);

extern SPS_Render_Context* SPS_Render_Context_Lookup(lua_State* state);
extern void SPS_Render_Context_Register(SPS_Render_Context* context, lua_State* state);

extern SPS_Page* SPS_Page_GetCurrent(SPS_Presentation* presentation);
extern SPS_Page* SPS_Page_Get(SPS_Presentation* presentation, int pageno);


extern SPS_Display* display; // XXX 

extern void SPS_Scene_Draw_Generic(SPS_Scene* scene, SPS_DRAW_PARAMETER_ENUM param_enum, SPS_Draw_Parameter* param, gboolean draw_bg);

extern void SPS_Sprite_Bind(SPS_Sprite* sprite);

extern void SPS_Flip_Function(SPS_Render_Context* context, const char* name);

// sps_presentation.c
extern SPS_Render_Context* SPS_Open_File(const char* path, gboolean parse);
extern void SPS_Page_Parse_All(SPS_Render_Context* context);

// sps_sprite.c
extern SPS_Sprite* SPS_Sprite_new();
extern void SPS_Sprite_Set_Surface(SPS_Sprite* sprite, SDL_Surface* surface);
extern SPS_Sprite* SPS_Sprite_new_WithSize(int width, int height);
extern void SPS_Sprite_Bind(SPS_Sprite* sprite);
void SPS_Sprite_Draw_Generic(SPS_Sprite* sprite, SPS_DRAW_PARAMETER_ENUM param_enum, SPS_Draw_Parameter* param);

extern SPS_Sprite* SPS_Sprite_from_String(TTF_Font* font, int style, SDL_Color color, gchar* string);
extern void SPS_Sprite_Free_Texture(SPS_Sprite* sprite);
extern void SPS_Sprite_Free_Surface(SPS_Sprite* sprite);
extern SPS_Sprite* SPS_Sprite_free(SPS_Sprite* sprite);

extern SPS_Flip_Vector* SPS_Flip_Driver_Lookup(const char* name);
extern SPS_Flip_Vector* SPS_Flip_Driver_Default();

#include "misc.h"

typedef enum {
	SPS_USEREVENT_PARSE_DONE,
	SPS_USEREVENT_WAIT_DONE,
	SPS_USEREVENT_ZOOM_DONE,
} SPS_USEREVENT;

#include "sps-displaybase.h"
extern void SPS_DisplayManager_AppendWindow(SpsDisplayBase* _parent, SpsDisplayBase* me);
extern void SPS_DisplayManager_RemoveWindow(SpsDisplayBase* me);
extern void SPS_DisplayManager_Raise(SpsDisplayBase* me);
extern void SPS_DisplayManager_MainLoop();
extern void SPS_DisplayManager_Init(SPS_Render_Context* _ctx);
extern TTF_Font* SPS_DisplayManager_GetFont(const char* name);

extern void SPS_DisplayManager_ShowTree();

extern void SPS_FPS_Reset();
extern void SPS_TextureCount_Reset();

#endif
